import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
import { Badge, Button, IconButton, ListItemDecorator, Menu, MenuItem, Sheet, Typography, useTheme, Box, Dropdown, MenuButton, } from "@mui/joy";
import { Dialog, DialogActions, Grow, Tooltip, useMediaQuery, } from "@mui/material";
import { useEffect, useRef, useState } from "react";
import { useHotkeys } from "react-hotkeys-hook";
import { Trans, useTranslation } from "react-i18next";
import { useSearchParams } from "react-router-dom";
import { Searchable } from "./searchable";
import { genericIcons } from "./utils";
import nerdUII18n from "../i18n";
/**
 *
 * @param {AdvancedSearchProps} props component props
 * @returns {ReactElement} NerdAdvancedSearch
 */
export function NerdAdvancedSearch({ definitions, open = false, setOpen, variant = "toolbar", buttonFaIcon = "fa-solid fa-magnifying-glass", buttonLabel, onSearch, onNewSearchPreset, searchMetadata, }) {
    const { t } = useTranslation("components", { i18n: nerdUII18n });
    const [_open, _setOpen] = useState(open);
    const [_variant, _setVariant] = useState(variant ?? localStorage.getItem("searchVariant"));
    const [_advancedMode, _setAdvancedMode] = useState(false);
    const theme = useTheme();
    const sm = useMediaQuery(theme.breakpoints.down("sm"));
    const _definitions = _advancedMode && searchMetadata ? searchMetadata : definitions;
    const [searchParams, setSearchParams] = useSearchParams();
    const filtersString = searchParams.get("filters");
    const filters = filtersString ? JSON.parse(filtersString) : [];
    const ref = useRef();
    useEffect(() => {
        if (sm) {
            _setVariant("dialog");
        }
        else {
            _setVariant("toolbar");
        }
    }, [sm]);
    useEffect(() => {
        localStorage.setItem("searchVariant", _variant);
    }, [_variant]);
    useEffect(() => {
        if (setOpen)
            setOpen(_open);
    }, [_open]);
    useEffect(() => {
        if (open != undefined)
            _setOpen(open);
    }, [open]);
    const go = () => {
        if (_variant === "dialog")
            _setOpen(false);
        setSearchParams((prevState) => {
            prevState.set("offset", "0");
            return prevState;
        });
        if (onSearch)
            onSearch({
                filters: searchParams.get("filters"),
                offset: searchParams.get("offset") ?? 0,
            });
    };
    useHotkeys(`enter`, go, { enableOnTags: ["INPUT", "TEXTAREA"] });
    const clearAll = () => {
        setSearchParams((prevState) => {
            prevState.set("offset", "0");
            prevState.delete("filters");
            return prevState;
        });
        if (onSearch)
            onSearch(undefined);
    };
    const handleAddField = (s) => () => {
        filters.push({
            name: s.name,
            comparison: "eq",
            value: s.type === "bool" ? true : "",
        });
        setSearchParams((prevState) => {
            prevState.set("filters", JSON.stringify(filters));
            prevState.set("offset", "0");
            return prevState;
        });
    };
    const goButton = onSearch ? (_jsx(Tooltip, { title: t("nerdAdvancedSearch.enter"), children: _jsx(Button, { endDecorator: _jsx("i", { className: "fa-sharp fa-solid fa-arrow-right" }), variant: "soft", size: "sm", sx: { background: _variant === "toolbar" ? "white" : undefined }, onClick: go, children: t("nerdAdvancedSearch.go") }) })) : ([]);
    const closeButton = (_jsx(Tooltip, { title: t("nerdAdvancedSearch.close_search"), children: _jsx(IconButton, { size: "sm", variant: "soft", onClick: () => _setOpen(false), children: _jsx("i", { className: "fa-solid fa-xmark" }) }) }));
    const moreOptionsButton = (_jsxs(Dropdown, { children: [_jsx(MenuButton, { slots: { root: IconButton }, slotProps: {
                    root: {
                        size: "sm",
                        variant: "soft",
                    },
                }, children: _jsx("i", { className: "fa-solid fa-ellipsis-vertical" }) }), _jsxs(Menu, { sx: {
                    zIndex: (theme) => theme.zIndex.modal,
                }, children: [onNewSearchPreset ? (_jsxs(MenuItem, { onClick: () => {
                            onNewSearchPreset();
                        }, children: [_jsx(ListItemDecorator, { sx: { color: "inherit" }, children: _jsx("i", { className: "fa-solid fa-download" }) }), t("nerdAdvancedSearch.save_as_preset")] })) : ([]), _jsxs(MenuItem, { onClick: () => {
                            _setVariant((s) => (s === "dialog" ? "toolbar" : "dialog"));
                        }, children: [_jsx(ListItemDecorator, { sx: { color: "inherit" }, children: _jsx("i", { className: _variant === "toolbar"
                                        ? "fa-solid fa-up-right-and-down-left-from-center"
                                        : "fa-solid fa-down-left-and-up-right-to-center" }) }), _variant === "toolbar"
                                ? t("nerdAdvancedSearch.expand")
                                : t("nerdAdvancedSearch.reduce")] }), (searchMetadata?.length ?? 0) > 0 ? (_jsx(Tooltip, { title: t("nerdAdvancedSearch.uses_search_metadata"), children: _jsxs(MenuItem, { onClick: () => {
                                _setAdvancedMode((s) => !s);
                            }, children: [_jsx(ListItemDecorator, { sx: { color: "inherit" }, children: _jsx("i", { className: "fa-solid fa-hammer" }) }), _advancedMode
                                    ? t("nerdAdvancedSearch.turn_dev_mode_off")
                                    : t("nerdAdvancedSearch.dev_mode")] }) })) : ([]), _jsxs(MenuItem, { onClick: () => {
                            clearAll();
                        }, children: [_jsx(ListItemDecorator, { sx: { color: "inherit" }, children: _jsx("i", { className: "fa-solid fa-trash" }) }), t("nerdAdvancedSearch.clear_all")] })] })] }));
    const addFieldList = (_jsx(Menu, { placement: "bottom-start", sx: {
            maxHeight: 400,
            overflow: "scroll",
            width: sm ? "calc(100% - 32px)" : undefined,
            zIndex: (theme) => theme.zIndex.modal,
        }, children: _definitions?.map((s) => (_jsxs(MenuItem, { onClick: handleAddField(s), children: [_jsx(ListItemDecorator, { sx: { color: "inherit" }, children: _jsx("i", { className: s.faIcon ?? genericIcons[s.type] }) }), s.label] }, s.id))) }));
    const addFieldIconButton = (_jsxs(Dropdown, { children: [_jsx(Tooltip, { title: t("nerdAdvancedSearch.add_new_field"), children: _jsx(MenuButton, { slots: { root: IconButton }, slotProps: {
                        root: {
                            size: "sm",
                        },
                    }, children: _jsx("i", { className: "fa-sharp fa-solid fa-plus" }) }) }), addFieldList] }));
    const toolbarVariant = (_jsx(Grow, { in: true, children: _jsxs(Sheet, { sx: {
                p: 1,
                borderRadius: 1,
                display: "flex",
                gap: 1,
                justifyContent: "space-between",
                position: "absolute",
                width: "100%",
                height: "100%",
                left: 0,
                top: 0,
                zIndex: 99,
            }, color: "primary", variant: "solid", children: [_jsxs(Box, { className: "noScrollBar", sx: {
                        display: "flex",
                        gap: 1,
                        flexGrow: 1,
                        overflowY: "hidden",
                        overflowX: "scroll",
                        scrollbarWidth: "none",
                        width: 0,
                    }, children: [closeButton, filters.map((f, i) => (_jsx(Box, { sx: { flexShrink: 0, display: "flex" }, children: _jsx(Searchable, { index: i, definition: [..._definitions, ...(searchMetadata ?? [])]?.find((m) => m.name === f.name), onDelete: (i) => console.log(i) }) }, f.name))), _jsx(Sheet, { variant: "solid", color: "primary", sx: {
                                position: "sticky",
                                right: 0,
                                zIndex: 99,
                                gap: 1,
                                display: "flex",
                            }, children: addFieldIconButton })] }), _jsxs(Box, { sx: { display: "flex", gap: 1 }, children: [goButton, moreOptionsButton] })] }) }));
    const dialogVariant = (_jsxs(Dialog, { open: true, onClose: () => _setOpen(false), maxWidth: !sm ? "sm" : undefined, fullWidth: true, fullScreen: sm, children: [_jsxs(Sheet, { color: "primary", variant: "solid", sx: {
                    p: 2,
                    color: "white",
                    display: "flex",
                    flexDirection: "column",
                    gap: 1,
                    flex: sm ? 1 : undefined,
                }, children: [_jsxs(Box, { sx: { display: "flex", justifyContent: "space-between" }, children: [_jsx(Typography, { level: "title-md", sx: { color: "inherit" }, startDecorator: _jsx("i", { className: "fa-solid fa-magnifying-glass" }), children: t("nerdAdvancedSearch.search") }), moreOptionsButton] }), _jsx(Typography, { level: "body-sm", sx: { color: "inherit" }, children: _jsx(Trans, { i18n: nerdUII18n, i18nKey: "components:nerdAdvancedSearch.description" }) }), _jsx(Box, { sx: {
                            display: "flex",
                            flexDirection: "column",
                            gap: 1,
                            alignItems: "stretch",
                        }, children: filters.map((f, i) => (_jsx(Searchable, { index: i, definition: _definitions?.find((m) => m.name === f.name), onDelete: (i) => console.log(i) }, f.name))) }), _jsx(Box, { sx: { display: "flex", justifyContent: "center" }, children: addFieldIconButton })] }), _jsxs(DialogActions, { children: [_jsx(Button, { size: "sm", onClick: () => _setOpen(false), variant: "soft", color: "neutral", children: t("nerdAdvancedSearch.close") }), goButton] })] }));
    return (_jsxs(Box, { ref: ref, children: [_jsx(Badge, { badgeContent: filters.length, size: "sm", color: "danger", badgeInset: "8%", children: sm ? (_jsx(IconButton, { variant: "solid", size: "sm", onClick: () => _setOpen((o) => !o), children: _jsx("i", { style: { fontSize: 14 }, className: buttonFaIcon }) })) : (_jsx(Button, { variant: "solid", size: "sm", onClick: () => _setOpen((o) => !o), endDecorator: !sm && filters.length ? (_jsx(Tooltip, { title: "Clear Preset", children: _jsx(Box, { sx: { p: 0.5 }, onClick: (e) => {
                                e.stopPropagation();
                                e.preventDefault();
                                clearAll();
                            }, children: _jsx("i", { style: { fontSize: 14, color: "white" }, className: "fa-solid fa-xmark" }) }) })) : undefined, startDecorator: _jsx("i", { style: { fontSize: 14 }, className: buttonFaIcon }), children: buttonLabel || t("nerdAdvancedSearch.search") })) }), _open ? (_jsxs(_Fragment, { children: [_variant === "toolbar" ? toolbarVariant : [], _variant === "dialog" ? dialogVariant : []] })) : ([])] }));
}
