import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
import { Chip, Divider, ListItemDecorator, Menu, MenuItem } from "@mui/joy";
import { AppBar, Box, Toolbar, Typography } from "@mui/material";
import { Fragment, useEffect, useRef, useState, } from "react";
/**
 *
 * @param {NerdNavigationBar} props Props
 * @returns {ReactElement} NerdNavigationBar
 */
export default function NerdNavigationBar(props) {
    const { leftItems, rightItems, chipSize, chipVariant, disableBottomDivider, navigationBarIndex, selectedChipVariant, } = props;
    const leftMenus = useRef([]);
    const [leftMenuIndex, setLeftMenuIndex] = useState(null);
    const leftAnchorRef = useRef([]);
    const leftChildrenMenus = useRef({});
    const rightMenus = useRef([]);
    const [rightMenuIndex, setRightMenuIndex] = useState(null);
    const rightAnchorRef = useRef([]);
    const rightChildrenMenus = useRef({});
    for (let i = 0; i < leftItems.length; i++) {
        if (!leftChildrenMenus.current[i])
            leftChildrenMenus.current[i] = [];
    }
    for (let i = 0; i < rightItems.length; i++) {
        if (!rightChildrenMenus.current[i])
            rightChildrenMenus.current[i] = [];
    }
    const [leftChildrenMenuIndex, setLeftChildrenMenuIndex] = useState(null);
    const [rightChildrenMenuIndex, setRightChildrenMenuIndex] = useState(null);
    const handleLeftMenuClick = (index) => {
        setLeftMenuIndex(index);
    };
    const handleRightMenuClick = (index) => {
        setRightMenuIndex(index);
    };
    const handleLeftChildrenMenuClick = (index) => {
        setLeftChildrenMenuIndex(index);
    };
    const handleRightChildrenMenuClick = (index) => {
        setRightChildrenMenuIndex(index);
    };
    const handleLeftMenuClose = () => {
        if (typeof leftMenuIndex === "number" &&
            typeof leftChildrenMenuIndex === "number" &&
            leftItems[leftMenuIndex] &&
            leftItems[leftMenuIndex].children) {
            const tempChildren = leftItems[leftMenuIndex].children;
            if (!tempChildren ||
                (tempChildren && !tempChildren[leftChildrenMenuIndex].children)) {
                setLeftMenuIndex(null);
            }
        }
        else {
            setLeftMenuIndex(null);
        }
    };
    const handleRightMenuClose = () => {
        if (typeof rightMenuIndex === "number" &&
            typeof rightChildrenMenuIndex === "number" &&
            rightItems[rightMenuIndex] &&
            rightItems[rightMenuIndex].children) {
            const tempChildren = rightItems[rightMenuIndex].children;
            if (!tempChildren ||
                (tempChildren && !tempChildren[rightChildrenMenuIndex].children)) {
                setRightMenuIndex(null);
            }
        }
        else {
            setRightMenuIndex(null);
        }
    };
    const handleLeftChildrenMenuClose = () => {
        setLeftMenuIndex(null);
        setLeftChildrenMenuIndex(null);
    };
    const handleRightChildrenMenuClose = () => {
        setRightMenuIndex(null);
        setRightChildrenMenuIndex(null);
    };
    useEffect(() => {
        if (leftChildrenMenuIndex === null &&
            typeof leftMenuIndex === "number" &&
            leftAnchorRef.current[leftMenuIndex]) {
            leftAnchorRef.current[leftMenuIndex].focus();
        }
    }, [leftChildrenMenuIndex]);
    useEffect(() => {
        if (rightChildrenMenuIndex === null &&
            typeof rightMenuIndex === "number" &&
            rightAnchorRef.current[rightMenuIndex]) {
            rightAnchorRef.current[rightMenuIndex].focus();
        }
    }, [rightChildrenMenuIndex]);
    /**
     *
     * @param {NavigationBarItem} item Item
     * @returns {boolean} Is selected
     */
    function isSelected(item) {
        if (item.selected) {
            return true;
        }
        else if (item.children) {
            let result = false;
            item.children.every((child) => {
                result = isSelected(child);
                if (result) {
                    return false;
                }
                else {
                    return true;
                }
            });
            return result;
        }
        else
            return false;
    }
    /**
     *
     * @param {NavigationBarItem} item item
     * @param {string} pathname pathname
     * @returns {boolean} is current path
     */
    function isCurrentPath(item, pathname) {
        if (item.to && "/" + item.to === pathname) {
            return true;
        }
        else if (item.children) {
            let result = false;
            item.children.every((child) => {
                result = isCurrentPath(child, pathname);
                if (result) {
                    return false;
                }
                else {
                    return true;
                }
            });
            return result;
        }
        else {
            return false;
        }
    }
    /**
     *
     * @param {object} props props
     * @param {NavigationBarItem} props.item item
     * @param {number} props.index index
     * @param {"left"|"right"} props.side side
     * @returns {ReactElement} navigationChip
     */
    function navigationChip(props) {
        const { index, item, side } = props;
        let variant = chipVariant ? chipVariant : "soft";
        if (isSelected(item))
            variant = selectedChipVariant ?? "soft";
        let endDecorator = null;
        if (item.endDecorator && !item.action && item.children) {
            endDecorator = (_jsxs(Box, { sx: { display: "flex", gap: 1 }, children: [item.endDecorator, _jsx("i", { className: "fa-sharp fa-solid fa-caret-down" })] }));
        }
        else if (item.endDecorator) {
            endDecorator = item.endDecorator;
        }
        else if (!item.endDecorator && !item.action && item.children) {
            endDecorator = _jsx("i", { className: "fa-sharp fa-solid fa-caret-down" });
        }
        return (_jsx(Chip, { variant: variant, color: isCurrentPath(item, location.pathname) || isSelected(item)
                ? "primary"
                : "neutral", size: chipSize ? chipSize : "sm", startDecorator: item.startDecorator ? item.startDecorator : null, endDecorator: endDecorator, onMouseEnter: () => {
                if (side === "left") {
                    if (typeof leftMenuIndex === "number") {
                        setLeftMenuIndex(index);
                    }
                    setLeftChildrenMenuIndex(null);
                }
                else {
                    if (typeof rightMenuIndex === "number") {
                        setRightMenuIndex(index);
                    }
                    setRightChildrenMenuIndex(null);
                }
            }, onClick: () => {
                if (item.action) {
                    side === "left" ? handleLeftMenuClose() : handleRightMenuClose();
                    item.action();
                }
                else if (item.children) {
                    if (side === "left") {
                        handleLeftMenuClick(index);
                    }
                    else {
                        handleRightMenuClick(index);
                    }
                }
            }, ref: (instance) => {
                if (instance) {
                    if (side === "left") {
                        leftMenus.current[index] = instance;
                    }
                    else {
                        rightMenus.current[index] = instance;
                    }
                }
            }, children: item.title }, index));
    }
    /**
     *
     * @param {object} props props
     * @param {NavigationBarItem} props.item item
     * @param {NavigationBarItem[]} props.siblings siblings
     * @returns {ReactElement} menuItemBox
     */
    function menuItemBox(props) {
        const { item, siblings } = props;
        let startDeco = false;
        if (siblings) {
            siblings.every((sibling) => {
                if (sibling.startDecorator) {
                    startDeco = true;
                    return false;
                }
                else {
                    return true;
                }
            });
        }
        let endDecorator = null;
        if (item.endDecorator && !item.action && item.children) {
            endDecorator = (_jsxs(Box, { sx: { display: "flex", gap: 1 }, children: [item.endDecorator, _jsx("i", { className: "fa-solid fa-angle-right" })] }));
        }
        else if (item.endDecorator) {
            endDecorator = item.endDecorator;
        }
        else if (!item.endDecorator && !item.action && item.children) {
            endDecorator = _jsx("i", { className: "fa-solid fa-angle-right" });
        }
        return (_jsxs(Box, { sx: {
                display: "flex",
                alignItems: "center",
                width: "100%",
            }, children: [startDeco ? (_jsx(ListItemDecorator, { children: item.startDecorator ? item.startDecorator : null })) : null, _jsxs(Box, { sx: {
                        display: "flex",
                        alignItems: "center",
                        justifyContent: "space-between",
                        width: "100%",
                    }, children: [_jsx(Typography, { color: isCurrentPath(item, location.pathname) || isSelected(item)
                                ? "primary"
                                : "neutral", sx: {
                                paddingRight: "10px",
                                paddingLeft: startDeco ? "0px" : "10px",
                            }, children: item.title }), _jsx(Box, { children: endDecorator })] })] }));
    }
    return (_jsxs(AppBar, { position: "fixed", sx: {
            background: (theme) => theme.palette.background.default,
            zIndex: navigationBarIndex ? navigationBarIndex : 1100,
            marginTop: "48px",
            boxShadow: "none",
            borderBottom: "solid 1px rgba(155,155,155,0.3)",
        }, color: "transparent", children: [_jsxs(Toolbar, { variant: "dense", sx: {
                    display: "flex",
                    justifyContent: "space-between",
                    gap: 1,
                    overflowY: "auto",
                }, children: [_jsx(Box, { sx: { display: "flex", gap: 1 }, children: leftItems.map((item, index) => {
                            return (_jsxs(Fragment, { children: [navigationChip({ item: item, index: index, side: "left" }), !item.action && item.children ? (_jsx(Menu, { id: `menu ${index}`, anchorEl: leftMenus.current[index], open: leftMenuIndex === index, onClose: handleLeftMenuClose, placement: "bottom-start", ref: (instance) => {
                                            if (instance)
                                                leftAnchorRef.current[index] = instance;
                                        }, sx: {
                                            borderRadius: 3,
                                            zIndex: navigationBarIndex
                                                ? navigationBarIndex + 1
                                                : 1101,
                                            background: (theme) => theme.palette.background.body,
                                        }, children: item.children.map((elem, elemIndex) => {
                                            return (_jsxs(_Fragment, { children: [_jsx(MenuItem, { onClick: () => {
                                                            if (elem.action) {
                                                                handleLeftMenuClose();
                                                                elem.action();
                                                            }
                                                            else {
                                                                handleLeftChildrenMenuClick(elemIndex);
                                                            }
                                                        }, onMouseEnter: () => {
                                                            if (!elem.children) {
                                                                setLeftChildrenMenuIndex(null);
                                                            }
                                                            else {
                                                                setLeftChildrenMenuIndex(elemIndex);
                                                            }
                                                        }, ref: (instance) => {
                                                            if (instance) {
                                                                if (leftChildrenMenus.current[index]) {
                                                                    leftChildrenMenus.current[index][elemIndex] =
                                                                        instance;
                                                                }
                                                                else {
                                                                    leftChildrenMenus.current[index] = [];
                                                                    leftChildrenMenus.current[index][elemIndex] =
                                                                        instance;
                                                                }
                                                            }
                                                        }, children: menuItemBox({
                                                            item: elem,
                                                            siblings: item.children,
                                                        }) }, elemIndex), !elem.action && elem.children ? (_jsx(Menu, { id: `menu child ${elemIndex}`, anchorEl: leftChildrenMenus.current[index][elemIndex], open: leftMenuIndex === index &&
                                                            leftChildrenMenuIndex === elemIndex, onClose: () => {
                                                            handleLeftChildrenMenuClose();
                                                        }, placement: "right-start", sx: {
                                                            borderRadius: 3,
                                                            zIndex: navigationBarIndex
                                                                ? navigationBarIndex + 2
                                                                : 1102,
                                                            background: (theme) => theme.palette.background.body,
                                                        }, children: elem.children.map((elem2, elem2Index) => {
                                                            return (_jsx(MenuItem, { onClick: () => {
                                                                    if (elem2.action) {
                                                                        handleLeftChildrenMenuClose();
                                                                        elem2.action();
                                                                    }
                                                                }, children: menuItemBox({
                                                                    item: elem2,
                                                                    siblings: elem.children,
                                                                }) }, elem2Index));
                                                        }) })) : (_jsx(_Fragment, {}))] }));
                                        }) })) : (_jsx(_Fragment, {}))] }, index));
                        }) }), _jsx(Box, { sx: { display: "flex", gap: 1 }, children: rightItems.map((item, index) => {
                            return (_jsxs(Fragment, { children: [navigationChip({ item: item, index: index, side: "right" }), !item.action && item.children ? (_jsx(Menu, { id: `menu ${index}`, anchorEl: rightMenus.current[index], open: rightMenuIndex === index, onClose: handleRightMenuClose, placement: "bottom-start", ref: (instance) => {
                                            if (instance)
                                                rightAnchorRef.current[index] = instance;
                                        }, sx: {
                                            borderRadius: 3,
                                            zIndex: navigationBarIndex
                                                ? navigationBarIndex + 1
                                                : 1101,
                                            background: (theme) => theme.palette.background.body,
                                        }, children: item.children.map((elem, elemIndex) => {
                                            return (_jsxs(_Fragment, { children: [_jsx(MenuItem, { onClick: () => {
                                                            if (elem.action) {
                                                                handleRightMenuClose();
                                                                elem.action();
                                                            }
                                                            else {
                                                                handleRightChildrenMenuClick(elemIndex);
                                                            }
                                                        }, onMouseEnter: () => {
                                                            if (!elem.children) {
                                                                setRightChildrenMenuIndex(null);
                                                            }
                                                            else {
                                                                setRightChildrenMenuIndex(elemIndex);
                                                            }
                                                        }, ref: (instance) => {
                                                            if (instance) {
                                                                if (rightChildrenMenus.current[index]) {
                                                                    rightChildrenMenus.current[index][elemIndex] =
                                                                        instance;
                                                                }
                                                                else {
                                                                    rightChildrenMenus.current[index] = [];
                                                                    rightChildrenMenus.current[index][elemIndex] =
                                                                        instance;
                                                                }
                                                            }
                                                        }, children: menuItemBox({
                                                            item: elem,
                                                            siblings: item.children,
                                                        }) }, elemIndex), !elem.action && elem.children ? (_jsx(Menu, { id: `menu child ${elemIndex}`, anchorEl: rightChildrenMenus.current[index][elemIndex], open: rightMenuIndex === index &&
                                                            rightChildrenMenuIndex === elemIndex, onClose: () => {
                                                            handleRightChildrenMenuClose();
                                                        }, placement: "right-start", sx: {
                                                            borderRadius: 3,
                                                            zIndex: navigationBarIndex
                                                                ? navigationBarIndex + 2
                                                                : 1102,
                                                            background: (theme) => theme.palette.background.body,
                                                        }, children: elem.children.map((elem2, elem2Index) => {
                                                            return (_jsx(MenuItem, { onClick: () => {
                                                                    if (elem2.action) {
                                                                        handleRightChildrenMenuClose();
                                                                        elem2.action();
                                                                    }
                                                                }, children: menuItemBox({
                                                                    item: elem2,
                                                                    siblings: elem.children,
                                                                }) }, elem2Index));
                                                        }) })) : (_jsx(_Fragment, {}))] }));
                                        }) })) : (_jsx(_Fragment, {}))] }, index));
                        }) })] }), disableBottomDivider ? null : _jsx(Divider, {})] }));
}
